@@ -85,6 +85,7 @@ int archinfo_show_fshead(cdico *dicofshead, int fsid)
 {
     char magic[FSA_SIZEOF_MAGIC+1];
     char fsbuf[FSA_MAX_FSNAMELEN];
+    u32 temp32;
     u64 temp64;
     u64 fsbytestotal;
     u64 fsbytesused;
@@ -122,10 +123,12 @@ int archinfo_show_fshead(cdico *dicofshead, int fsid)
     if (dico_get_string(dicofshead, 0, FSYSHEADKEY_ORIGDEV, fsorigdev, sizeof(fsorigdev))<0)
         snprintf(fsorigdev, sizeof(fsorigdev), "<unknown>");
 
-    // filesystem uuid: maybe an ntfs uuid or an unix uuid
+    // filesystem uuid: maybe an ntfs uuid, fat serial or an unix uuid
     snprintf(fsuuid, sizeof(fsuuid), "<none>");
     if (dico_get_u64(dicofshead, 0, FSYSHEADKEY_NTFSUUID, &temp64)==0)
         snprintf(fsuuid, sizeof(fsuuid), "%016llX", (long long unsigned int)temp64);
+    else if (dico_get_u32(dicofshead, 0, FSYSHEADKEY_FSVFATSERIAL, &temp32)==0)
+        snprintf(fsuuid, sizeof(fsuuid), "%08lX", (long unsigned int)temp32);
     else if (dico_get_string(dicofshead, 0, FSYSHEADKEY_FSUUID, buffer, sizeof(buffer))==0 && strlen(buffer)==36)
         snprintf(fsuuid, sizeof(fsuuid), "%s", buffer);
 
