@@ -52,13 +52,14 @@ int jfs_mkfs(cdico *d, char *partition, char *fsoptions, char *mkfslabel, char *
     // ---- set the advanced filesystem settings from the dico
     memset(options, 0, sizeof(options));
 
-    strlcatf(options, sizeof(options), " %s ", fsoptions);
-
     if (strlen(mkfslabel) > 0)
         strlcatf(options, sizeof(options), " -L '%s' ", mkfslabel);
     else if (dico_get_string(d, 0, FSYSHEADKEY_FSLABEL, buffer, sizeof(buffer))==0 && strlen(buffer)>0)
         strlcatf(options, sizeof(options), " -L '%s' ", buffer);
 
+    // ---- mkfsopt from command line
+    strlcatf(options, sizeof(options), " %s ", fsoptions);
+
     if (exec_command(command, sizeof(command), &exitst, NULL, 0, NULL, 0, "jfs_mkfs -q %s %s", options, partition)!=0 || exitst!=0)
     {   errprintf("command [%s] failed\n", command);
         return -1;
@@ -73,7 +74,7 @@ int jfs_mkfs(cdico *d, char *partition, char *fsoptions, char *mkfslabel, char *
 
     if (options[0])
     {
-        if (exec_command(command, sizeof(command), &exitst, NULL, 0, NULL, 0, "jfs_tune %s %s", partition, options)!=0 || exitst!=0)
+        if (exec_command(command, sizeof(command), &exitst, NULL, 0, NULL, 0, "jfs_tune %s %s", options, partition)!=0 || exitst!=0)
         {   errprintf("command [%s] failed\n", command);
             return -1;
         }
