@@ -185,18 +185,22 @@ struct xfs_sb
 #define XFS_SB_FEAT_RO_COMPAT_FINOBT      (1 << 0)  /* free inode btree */
 #define XFS_SB_FEAT_RO_COMPAT_RMAPBT      (1 << 1)  /* reverse map btree */
 #define XFS_SB_FEAT_RO_COMPAT_REFLINK     (1 << 2)  /* reflinked files */
+#define XFS_SB_FEAT_RO_COMPAT_INOBTCNT    (1 << 3)  /* inobt block counts */
 #define XFS_SB_FEAT_INCOMPAT_FTYPE        (1 << 0)  /* filetype in dirent */
 #define XFS_SB_FEAT_INCOMPAT_SPINODES     (1 << 1)  /* sparse inode chunks */
 #define XFS_SB_FEAT_INCOMPAT_META_UUID    (1 << 2)  /* metadata UUID */
+#define XFS_SB_FEAT_INCOMPAT_BIGTIME      (1 << 3)  /* large timestamps */
 
 // features supported by the current fsarchiver version
 #define FSA_XFS_FEATURE_COMPAT_SUPP       (u64)(0)
 #define FSA_XFS_FEATURE_RO_COMPAT_SUPP    (u64)(XFS_SB_FEAT_RO_COMPAT_FINOBT|\
                                                 XFS_SB_FEAT_RO_COMPAT_RMAPBT|\
-                                                XFS_SB_FEAT_RO_COMPAT_REFLINK)
+                                                XFS_SB_FEAT_RO_COMPAT_REFLINK|\
+                                                XFS_SB_FEAT_RO_COMPAT_INOBTCNT)
 #define FSA_XFS_FEATURE_INCOMPAT_SUPP     (u64)(XFS_SB_FEAT_INCOMPAT_FTYPE|\
                                                 XFS_SB_FEAT_INCOMPAT_SPINODES|\
-                                                XFS_SB_FEAT_INCOMPAT_META_UUID)
+                                                XFS_SB_FEAT_INCOMPAT_META_UUID|\
+                                                XFS_SB_FEAT_INCOMPAT_BIGTIME)
 #define FSA_XFS_FEATURE_LOG_INCOMPAT_SUPP (u64)(0)
 
 #endif // __FS_XFS_H__
